"""
# File       : 用户验证.py
# Time       ：2024/8/20 下午5:31
# Author     ：xuewei zhang
# Email      ：shuiheyangguang@gmail.com
# version    ：python 3.12
# Description：
"""
from sqlalchemy.orm import selectinload
from fastapi import APIRouter, Depends
from sqlalchemy.future import select
from sqlalchemy.ext.asyncio import AsyncSession

from user_auth_zxw.db.models import User, Role, App
from user_auth_zxw.db.get_db import get_db
from user_auth_zxw.apis.schemas import 请求_分配或创建角色, 返回_分配或创建角色

from app_tools_zxw.Errors.api_errors import HTTPException_AppToolsSZXW, ErrorCode
from app_tools_zxw.Funcs.fastapi_logger import logger

router = APIRouter(prefix="/roles")


async def delete_role(user_id: int, role_name: str, db: AsyncSession = Depends(get_db)):
    # 1. 查询用户
    user_result = await db.execute(
        select(User).options(selectinload(User.roles)).filter(User.id == user_id)
    )
    user = user_result.scalar_one_or_none()
    if not user:
        raise HTTPException_AppToolsSZXW(
            error_code=ErrorCode.用户未找到,
            detail="用户未找到",
            http_status_code=404
        )

    # 2. 查询角色
    role = next((role for role in user.roles if role.name == role_name), None)
    if not role:
        raise HTTPException_AppToolsSZXW(
            error_code=ErrorCode.角色未找到,
            detail="角色未找到",
            http_status_code=404
        )

    # 3. 解除用户与角色关联
    if role in user.roles:
        user.roles.remove(role)
        try:
            await db.commit()
            return {"status": True, "message": "角色删除成功"}
        except Exception as e:
            await db.rollback()
            logger.error(f"角色删除失败: {e}")
            raise HTTPException_AppToolsSZXW(
                error_code=ErrorCode.角色未找到,
                detail="角色删除失败",
                http_status_code=500
            )
    else:
        raise HTTPException_AppToolsSZXW(
            error_code=ErrorCode.角色未找到,
            detail="用户没有该角色",
            http_status_code=400
        )


async def add_new_role(request: 请求_分配或创建角色, db: AsyncSession = Depends(get_db)):
    """
    本函数主要作用：给用户分配角色,
                如果用户不存在则返回404,
                如果角色不存在则创建角色,
                如果应用不存在则创建应用,
                如果用户和角色都存在则给用户分配角色
    """
    # Eagerly load roles to avoid lazy loading issues
    # 懒加载原理是：当我们访问一个对象的属性时，如果这个属性没有被加载，那么sqlalchemy会自动发起一个sql查询，加载这个属性。
    # 这种加载方式是懒加载，也就是说，当我们访问一个属性时，才会加载这个属性。
    # 但是这种加载方式有一个问题，就是如果我们在一个session中加载了一个对象，然后关闭了这个session，那么这个对象的属性就不能被访问了。

    # 1. 查询用户
    result_user = await db.execute(
        select(User).options(selectinload(User.roles)).filter(User.id == request.user_id)
    )
    user = result_user.scalar_one_or_none()

    # 2. 查询app name
    result_app = await db.execute(select(App).filter(App.name == request.app_name))
    app = result_app.scalar_one_or_none()  # scalar_one_or_none()作用是返回一个结果，如果没有结果则返回None

    # 3. 新增app name , 如果不存在
    if not app:
        app = App(name=request.app_name)
        db.add(app)
        await db.flush()
        await db.refresh(app)

    # 4. 查询role name
    result_role = await db.execute(select(Role).filter(
        Role.name == request.role_name,
        Role.app == app))
    role = result_role.scalar_one_or_none()

    # 5. 新增role name, 如果不存在.
    if not role:
        role = Role(name=request.role_name, app_id=app.id)
        db.add(role)
        try:
            await db.flush()
            await db.refresh(role)
        except Exception as e:
            logger.error(f"角色创建失败: {e}")
            raise HTTPException_AppToolsSZXW(
                error_code=ErrorCode.角色创建失败,
                detail="角色创建失败",
                http_status_code=500
            )

    # 6. 关联user和role
    if user and role:
        user.roles.append(role)
        await db.commit()
        return {"status": True, "message": "角色分配成功"}

    if not user:
        raise HTTPException_AppToolsSZXW(
            error_code=ErrorCode.用户未找到,
            detail="用户未找到",
            http_status_code=404
        )

    raise HTTPException_AppToolsSZXW(
        error_code=ErrorCode.角色未找到,
        detail="角色未找到",
        http_status_code=404
    )


@router.post("/assign-role/", response_model=返回_分配或创建角色)
async def __分配或创建角色__(request: 请求_分配或创建角色):
    return await add_new_role(request)



"""
# File       : __init__.py.py
# Time       ：2024/9/24 08:23
# Author     ：xuewei zhang
# Email      ：shuiheyangguang@gmail.com
# version    ：python 3.12
# Description：
"""
from .main import router
from .apis import schemas
from .apis.api_用户权限_增加 import add_new_role, delete_role
from .apis.api_用户权限_验证 import require_role
from .SDK_jwt.jwt import create_jwt_token, get_current_user, check_jwt_token, oauth2_scheme
from .SDK_jwt.jwt_刷新管理 import create_refresh_token



"""
# File       : __init__.py.py
# Time       ：2024/9/17 上午3:57
# Author     ：xuewei zhang
# Email      ：shuiheyangguang@gmail.com
# version    ：python 3.12
# Description：
"""
from . import api_注册登录



"""
# File       : api_刷新JWT.py
# Time       ：2024/8/22 18:00
# Author     ：xuewei zhang
# Email      ：shuiheyangguang@gmail.com
# version    ：python 3.12
# Description：
"""
from fastapi import APIRouter, Depends, HTTPException
from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy.future import select
from user_auth_zxw.db.get_db import get_db
from user_auth_zxw.db.models import User
from user_auth_zxw.SDK_jwt import (create_jwt_token,
                                   create_refresh_token,
                                   RefreshToken,
                                   get_current_user)
from user_auth_zxw.SDK_jwt.jwt import check_jwt_token, oauth2_scheme
from user_auth_zxw.apis.schemas import (请求_更新Token,
                                        返回_更新Token,
                                        请求_检查Token_from_body,
                                        Payload)

router = APIRouter(prefix="/token")


@router.post("/get-current-user/", response_model=Payload)
async def 获取当前用户(user: User = Depends(get_current_user)) -> Payload:
    return user.to_payload()


@router.post("/refresh-token/", response_model=返回_更新Token)
async def 更新token(refresh_token: 请求_更新Token, db: AsyncSession = Depends(get_db)):
    refresh_token = refresh_token.refresh_token
    # Check if the refresh token is valid
    result = await db.execute(select(RefreshToken).filter(RefreshToken.token == refresh_token))
    token = result.scalar_one_or_none()

    if token is None or token.is_expired():
        raise HTTPException(status_code=401, detail="Invalid or expired refresh token")

    user = await db.get(User, token.user_id)
    if user is None:
        raise HTTPException(status_code=401, detail="User not found")

    # 创建payload
    payload: Payload = user.to_payload()

    # Generate a new JWT token
    access_token = create_jwt_token(payload=payload.model_dump())

    # Optionally, generate a new refresh token
    new_refresh_token = create_refresh_token(user.id, db)
    await db.delete(token)  # Delete the old refresh token
    await db.commit()

    return {"access_token": access_token, "refresh_token": new_refresh_token.token}


async def validate_token(access_token: str) -> Payload:
    payload = check_jwt_token(access_token)
    if payload is None:
        raise HTTPException(status_code=401, detail="Invalid token")
    return payload


@router.post("/check-token-from-body/", response_description="返回payload")
async def 检查token_from_body(access_token: 请求_检查Token_from_body):
    access_token = access_token.access_token
    return await validate_token(access_token)


@router.post("/check-token-from-header/", response_description="返回payload")
async def 检查token_from_header(access_token: str = Depends(oauth2_scheme)):
    return await validate_token(access_token)



"""
# File       : api_注册.py
# Time       ：2024/8/22 18:41
# Author     ：xuewei zhang
# Email      ：shuiheyangguang@gmail.com
# version    ：python 3.12
# Description：
"""
from sqlalchemy.orm import selectinload
from fastapi import APIRouter, Depends
from fastapi.security import OAuth2PasswordRequestForm
from sqlalchemy.future import select
from sqlalchemy.ext.asyncio import AsyncSession
from user_auth_zxw.db.models import User
from user_auth_zxw.db.get_db import get_db
from user_auth_zxw.apis.api_登录注册.login import login_user_async
from user_auth_zxw.apis.api_用户权限_增加 import 分配或创建角色
from app_tools_zxw.Errors.api_errors import HTTPException_AppToolsSZXW, ErrorCode
from app_tools_zxw.msvc_user_auth.schemas import (返回_login,
                                                  请求_分配或创建角色,
                                                  请求_账号密码_注册,
                                                  请求_账号密码_登录)

router = APIRouter(prefix="/account/normal")


@router.post("/register/", response_model=返回_login)
async def 注册(user_create: 请求_账号密码_注册, db: AsyncSession = Depends(get_db)):
    # 检查用户名是否已存在
    result = await db.execute(select(User).filter(User.username == user_create.username))
    user = result.scalar_one_or_none()
    if user:
        raise HTTPException_AppToolsSZXW(error_code=ErrorCode.用户名已注册,
                                         detail="Username already registered",
                                         http_status_code=400)
    # 创建新用户并保存到数据库
    new_user = User(
        username=user_create.username,
        phone=user_create.username,
        email=user_create.username,
        openid=user_create.username,
        hashed_password=User.hash_password(user_create.password),
        notes="phone register"
    )
    db.add(new_user)
    await db.commit()
    await db.refresh(new_user)

    # 增加初始权限
    user_info = 请求_分配或创建角色(user_id=new_user.id, role_name=user_create.role_name, app_name=user_create.app_name)
    await 分配或创建角色(user_info, db)
    #
    return await login_user_async(new_user, db)


@router.post("/login/", response_model=返回_login)
async def 登录(info: 请求_账号密码_登录, db: AsyncSession = Depends(get_db)):
    result = await db.execute(select(User).options(selectinload(User.roles)).filter(User.username == info.username))
    user = result.scalar_one_or_none()
    print("apis/api_登录注册/api_注册.py: 登录(): user = ", user)

    if user is None or not user.verify_password(info.password):
        raise HTTPException_AppToolsSZXW(error_code=ErrorCode.无效的用户名或密码,
                                         detail="Invalid username or password",
                                         http_status_code=400)
    return await login_user_async(user, db)


@router.post("/login-form/", response_model=返回_login)
async def 登录_Form数据(login_info: OAuth2PasswordRequestForm = Depends(), db: AsyncSession = Depends(get_db)):
    #
    username = login_info.username
    password = login_info.password
    #
    result = await db.execute(select(User).options(selectinload(User.roles)).filter(User.username == username))
    user = result.scalar_one_or_none()
    print("apis/api_登录注册/api_注册.py: 登录(): user = ", user)

    if user is None or not user.verify_password(password):
        raise HTTPException_AppToolsSZXW(error_code=ErrorCode.无效的用户名或密码,
                                         detail="Invalid username or password",
                                         http_status_code=400)
    return await login_user_async(user, db)



"""
# File       : 微信一键登录.py
# Time       ：2024/8/20 下午5:24
# Author     ：xuewei zhang
# Email      ：shuiheyangguang@gmail.com
# version    ：python 3.12
# Description：
"""
from fastapi import APIRouter, Depends
from sqlalchemy.future import select
from sqlalchemy.ext.asyncio import AsyncSession
from user_auth_zxw.db.models import User
from user_auth_zxw.db.get_db import get_db
from user_auth_zxw.config import WeChatPub
from user_auth_zxw.apis.api_登录注册.login import login_user_async
from user_auth_zxw.apis.api_用户权限_增加 import 分配或创建角色
from user_auth_zxw.apis.schemas import 返回_login, 请求_获取_登录二维码URL, 返回_获取_登录二维码URL, \
    请求_分配或创建角色

from app_tools_zxw.SDK_微信.SDK_微信公众号或网站应用.获取_openid和access_token import get_access_token_and_openid_async
from app_tools_zxw.SDK_微信.SDK_微信公众号或网站应用.获取_登录二维码URL import get_qrcode_url
from app_tools_zxw.Errors.api_errors import HTTPException_AppToolsSZXW, ErrorCode

router = APIRouter(prefix="/account/wechat/qr-login", tags=["微信一键登录"])


@router.post("/get-qrcode", response_model=返回_获取_登录二维码URL)
async def 获取_登录二维码URL(url: 请求_获取_登录二维码URL):
    """
    :param WECHAT_REDIRECT_URI: 与网页授权获取用户信息里的回调域名一致
    :return:
    """
    try:
        qrcode_url = get_qrcode_url(url.WECHAT_REDIRECT_URI, WeChatPub.scope_qrcode_login)
        return 返回_获取_登录二维码URL(qr_code_url=qrcode_url)
    except Exception as e:
        raise HTTPException_AppToolsSZXW(error_code=ErrorCode.生成二维码失败,
                                         detail="Failed to generate WeChat QR code URL",
                                         http_status_code=500)


@router.post("/login/", response_model=返回_login)
async def 一键登录(code: str, app_name: str, db: AsyncSession = Depends(get_db)):
    """
    逻辑：通过code获取openid，然后判断是否存在用户，不存在则创建用户
    :param code: 微信登录code
    :param app_name: app名称
    :param db:
    :return:
    """
    access_key, openid = await get_access_token_and_openid_async(code,
                                                                 WeChatPub.app_id,
                                                                 WeChatPub.app_secret)

    if openid:
        result = await db.execute(select(User).filter(User.openid == openid))
        user = result.scalar_one_or_none()
        if not user:
            user = User(openid=openid,
                        username=openid,
                        phone=openid,
                        email=openid,
                        nickname="WeChatPub User")
            db.add(user)
            await db.commit()
            await db.refresh(user)
            # 增加初始权限
            user_info = 请求_分配或创建角色(user_id=user.id,
                                            role_name="l0",
                                            app_name=app_name)
            await 分配或创建角色(user_info, db)
        #
        return await login_user_async(user, db)

    raise HTTPException_AppToolsSZXW(error_code=ErrorCode.微信登录失败,
                                     detail="Login failed",
                                     http_status_code=400)



import random
from sqlalchemy.orm import selectinload
from fastapi import APIRouter, Depends
from sqlalchemy.future import select
from sqlalchemy.ext.asyncio import AsyncSession
from user_auth_zxw.db.models import User
from user_auth_zxw.db.get_db import get_db
from user_auth_zxw.apis.api_登录注册.login import login_user_async
from user_auth_zxw.apis.api_用户权限_增加 import 分配或创建角色
from user_auth_zxw.apis.schemas import (返回_login,
                                        请求_分配或创建角色,
                                        请求_手机邮箱_注册,
                                        请求_手机邮箱_登录)
from datetime import datetime, timedelta

from user_auth_zxw.config import AliyunSMS
from user_auth_zxw.SDK_jwt import (get_current_user)

from app_tools_zxw.SDK_阿里云.SMS_发送短信v2 import SMS阿里云
from app_tools_zxw.Errors.api_errors import HTTPException_AppToolsSZXW, ErrorCode


# 用于存储验证码的字典
verification_codes = {}


def store_verification_code(phone: str, code: str):
    verification_codes[phone] = {
        "code": code,
        "expiry": datetime.now() + timedelta(minutes=5)
    }


def verify_code(phone: str, code: str) -> bool:
    if phone not in verification_codes:
        return False
    stored_code = verification_codes[phone]
    if datetime.now() > stored_code["expiry"]:
        del verification_codes[phone]
        return False
    if stored_code["code"] != code:
        return False
    del verification_codes[phone]
    return True


sms = SMS阿里云(AliyunSMS.ali_access_key_id, AliyunSMS.ali_access_key_secret)
router = APIRouter(prefix="/account/phone")


@router.post("/send-verification-code/")
async def 发送验证码(phone: str):
    verification_code = ''.join([str(random.randint(0, 9)) for _ in range(4)])  # 生成6位随机数字验证码
    result = await sms.发送_验证码_async(phone, verification_code)
    if result is None:
        store_verification_code(phone, verification_code)
        return {"message": "验证码已发送"}
    else:
        raise HTTPException_AppToolsSZXW(error_code=ErrorCode.验证码发送失败,
                                         detail="Failed to send verification code",
                                         http_status_code=500)


@router.post("/register-phone/", response_model=返回_login)
async def 注册_手机(user_create: 请求_手机邮箱_注册, db: AsyncSession = Depends(get_db)):
    # 检查手机号是否已存在
    result = await db.execute(select(User).filter(User.phone == user_create.phone))
    user = result.scalar_one_or_none()
    if user:
        raise HTTPException_AppToolsSZXW(error_code=ErrorCode.手机号已注册,
                                         detail="Phone number already registered",
                                         http_status_code=400)
    # 验证验证码
    if not verify_code(user_create.phone, user_create.sms_code):
        raise HTTPException_AppToolsSZXW(error_code=ErrorCode.验证码验证失败,
                                         detail="Invalid verification code",
                                         http_status_code=400)
    # 创建新用户并保存到数据库
    new_user = User(
        username=user_create.phone,
        email=user_create.email if user_create.email else None,
        phone=user_create.phone,
        hashed_password=User.hash_password(user_create.sms_code),
        notes="phone register"
    )
    db.add(new_user)
    await db.commit()
    await db.refresh(new_user)

    # 增加初始权限
    user_info = 请求_分配或创建角色(
        user_id=new_user.id,
        role_name=user_create.role_name,
        app_name=user_create.app_name)
    await 分配或创建角色(user_info, db)
    return await login_user_async(new_user, db)


@router.post("/login-phone/", response_model=返回_login)
async def 登录_手机(info: 请求_手机邮箱_登录, db: AsyncSession = Depends(get_db)):
    result = await db.execute(select(User).options(selectinload(User.roles)).filter(User.phone == info.phone))
    user = result.scalar_one_or_none()
    if user is None or not verify_code(info.phone, info.sms_code):
        raise HTTPException_AppToolsSZXW(error_code=ErrorCode.无效的手机号或验证码,
                                         detail="Invalid phone number or SMS code",
                                         http_status_code=400)
    return await login_user_async(user, db)


@router.post("/change-phone/", response_model=返回_login)
async def 更换绑定手机号(new_phone: str,
                         sms_code: str,
                         db: AsyncSession = Depends(get_db),
                         user: User = Depends(get_current_user)):
    user: User | None = await db.get(User, user.id)
    if user is None or not verify_code(new_phone, sms_code):
        raise HTTPException_AppToolsSZXW(error_code=ErrorCode.无效的手机号或验证码,
                                         detail="Invalid user or SMS code",
                                         http_status_code=400)
    user.phone = new_phone
    await db.commit()
    await db.refresh(user)
    return await login_user_async(user, db)



"""
# File       : jwt_token刷新.py
# Time       ：2024/8/22 17:57
# Author     ：xuewei zhang
# Email      ：shuiheyangguang@gmail.com
# version    ：python 3.12
# Description：在用户登录成功时，生成JWT和刷新令牌，并将刷新令牌存储在数据库中。
"""
import secrets
from datetime import datetime, timedelta
from user_auth_zxw.SDK_jwt.db_model.model import RefreshToken
from sqlalchemy.ext.asyncio import AsyncSession


def create_refresh_token(user_id: int, db: AsyncSession, expires_delta: timedelta = timedelta(days=7)):
    token = secrets.token_hex(32)
    expires_at = datetime.utcnow() + expires_delta
    refresh_token = RefreshToken(token=token, user_id=user_id, expires_at=expires_at)
    db.add(refresh_token)
    return refresh_token



"""
# File       : 测试短信验证码.py
# Time       ：2024/9/17 09:46
# Author     ：xuewei zhang
# Email      ：shuiheyangguang@gmail.com
# version    ：python 3.12
# Description：
"""
import asyncio
from app_tools_zxw.SDK_阿里云.SMS_发送短信v2 import SMS阿里云
from user_auth_zxw.config import AliyunSMS

sms = SMS阿里云(AliyunSMS.ali_access_key_id, AliyunSMS.ali_access_key_secret)
re = asyncio.run(sms.发送_验证码_async('17512541044', "12345"))
# re = sms.发送短信验证码SMS('17512541044', "12345")
print(re)
